# -*- coding: utf-8; -*-
#
# (c) 2004-2007 Linbox / Free&ALter Soft, http://linbox.com
# (c) 2007 Mandriva, http://www.mandriva.com/
#
# $Id: __init__.py 7729 2010-11-19 17:07:15Z cdelfosse $
#
# This file is part of Mandriva Management Console (MMC).
#
# MMC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# MMC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with MMC; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
Class to manage imaging mmc-agent api
imaging plugin
"""

import logging

from mmc.agent import PluginManager
from mmc.support.mmctools import ContextMakerI, SecurityContext
from mmc.plugins.imaging.config import ImagingConfig
from mmc.plugins.imaging.profile import ImagingProfile
from mmc.plugins.imaging.computer import InventoryComputers
from mmc.plugins.imaging.imaging import ComputerImagingImaging
from mmc.plugins.imaging.pulse import ImagingPulse2Manager
from mmc.plugins.imaging.functions import ImagingRpcProxy
from pulse2.managers.profile import ComputerProfileManager
from mmc.plugins.base.computers import ComputerManager
from pulse2.managers.imaging import ComputerImagingManager
from pulse2.managers.pulse import Pulse2Manager
from pulse2.database.imaging import ImagingDatabase

VERSION = "1.3.0"
APIVERSION = "0:0:0"
REVISION = int("$Rev: 7729 $".split(':')[1].strip(' $'))

NOAUTHNEEDED = ['computerRegister',
                'imagingServerRegister',
                'getComputerByMac',
                'imageRegister',
                'imageUpdate',
                'logClientAction',
                'injectInventory',
                'getDefaultMenuForRegistering',
                'linkImagingServerToLocation',
                'computerChangeDefaultMenuItem']


def getVersion():
    return VERSION


def getApiVersion():
    return APIVERSION


def getRevision():
    return REVISION


def activate():
    """
    Read the plugin configuration, initialize it, and run some tests to ensure
    it is ready to operate.
    """
    logger = logging.getLogger()
    config = ImagingConfig("imaging")

    if config.disabled:
        logger.warning("Plugin imaging: disabled by configuration.")
        return False

    # Initialize imaging database
    if not ImagingDatabase().activate(config):
        logger.warning("Plugin imaging: an error occurred during the database initialization")
        return False

    # register ImagingProfile in ComputerProfileManager but only as a client
    ComputerProfileManager().register("imaging", ImagingProfile)

    ComputerImagingManager().register("imaging", ComputerImagingImaging)

    Pulse2Manager().register('imaging', ImagingPulse2Manager)

    ComputerManager().register('imaging', InventoryComputers)

    return True

def activate_2():
    """
    Check that the MMC pulse2 plugin is enabled
    """
    if not PluginManager().isEnabled('pulse2'):
        ret = False
        logging.getLogger().error("Plugin imaging: plugin is disabled because the pulse2 plugin is not available")
    else:
        ret = True
    return ret

class ContextMaker(ContextMakerI):

    def getContext(self):
        s = SecurityContext()
        s.userid = self.userid
        return s

class RpcProxy(ImagingRpcProxy):
    pass
